@@ -18,15 +18,15 @@
 
 #include "kalgebramobile.h"
 
+#include <analitzaplot/plotsmodel.h>
 #include <analitza/variables.h>
-#include <analitzagui/functionsmodel.h>
-#include <analitzagui/variablesmodel.h>
-#include "analitzawrapper.h"
+#include <analitza/expression.h>
 
-#include <QDeclarativeContext>
-#include "graph2dmobile.h"
+#include <qdeclarative.h>
 #include "pluginsmodel.h"
 
+using namespace Analitza;
+
 KAlgebraMobile* KAlgebraMobile::s_self=0;
 KAlgebraMobile* KAlgebraMobile::self() { return s_self; }
 
@@ -36,63 +36,48 @@ KAlgebraMobile::KAlgebraMobile(QObject* parent)
 	Q_ASSERT(s_self==0);
 	s_self=this;
 	
-	qmlRegisterType<PluginsModel>("org.kde.analitza", 1, 0, "PluginsModel");
-	qmlRegisterType<AnalitzaWrapper>("org.kde.analitza", 1, 0, "Analitza");
-	qmlRegisterType<ExpressionWrapper>("org.kde.analitza", 1, 0, "Expression");
-	qmlRegisterType<FunctionsModel>("org.kde.analitza", 1, 0, "FunctionsModel");
-	qmlRegisterType<Graph2DMobile>("org.kde.analitza", 1, 0, "Graph2DView");
-	qmlRegisterType<VariablesModel>("org.kde.analitza", 1, 0, "VariablesModel");
-	qmlRegisterInterface<Analitza::Variables*>("Analitza::Variables");
-// 	global.setProperty("VariablesModel", varsmodel, QScriptValue::Undeletable|QScriptValue::ReadOnly);
+	qmlRegisterType<PluginsModel>("org.kde.kalgebra.mobile", 1, 0, "PluginsModel");
+	qmlRegisterType<QAbstractItemModel>();
 }
 
-FunctionsModel* KAlgebraMobile::functionsModel()
+PlotsModel* KAlgebraMobile::functionsModel()
 {
 	if(!m_functionsModel) {
-		m_functionsModel = new FunctionsModel(this);
-		m_functionsModel->setResolution(500);
-		connect(m_functionsModel, SIGNAL(functionRemoved(QString)), SLOT(functionRemoved(QString)));
-		connect(m_functionsModel, SIGNAL(functionModified(QString,Analitza::Expression)), SLOT(functionModified(QString,Analitza::Expression)));
+		m_functionsModel = new Analitza::PlotsModel(this);
+		connect(m_functionsModel, SIGNAL(rowsRemoved(QModelIndex,int,int)), SLOT(functionRemoved(QModelIndex,int,int)));
+		connect(m_functionsModel, SIGNAL(rowsInserted(QModelIndex,int,int)), SLOT(functionInserted(QModelIndex,int,int)));
+		connect(m_functionsModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)), SLOT(functionModified(QModelIndex, QModelIndex)));
 	}
 	
 	return m_functionsModel;
 }
 
-void KAlgebraMobile::functionRemoved(const QString& name)
+void KAlgebraMobile::functionInserted(const QModelIndex& parent, int start, int end)
 {
-	m_vars->remove(name);
+	Q_ASSERT(!parent.isValid());
+	for(int i=start; i<=end; i++) {
+		QModelIndex idx = functionsModel()->index(i, 1);
+		m_vars->modify(idx.sibling(i,0).data().toString(), Analitza::Expression(idx.data().toString()));
+	}
 }
 
-void KAlgebraMobile::functionModified(const QString& name, const Analitza::Expression& exp)
+void KAlgebraMobile::functionRemoved(const QModelIndex& parent, int start, int end)
 {
-	m_vars->modify(name, exp);
+	Q_ASSERT(!parent.isValid());
+	for(int i=start; i<=end; i++) {
+		QModelIndex idx = functionsModel()->index(i);
+		m_vars->remove(functionsModel()->data(idx, Qt::DisplayRole).toString());
+	}
+}
+
+void KAlgebraMobile::functionModified(const QModelIndex& idxA, const QModelIndex& idxB)
+{
+	if(idxB.column()==1) {
+		for(int i=idxA.row(); i<idxB.row(); ++i) {
+			QModelIndex idx = functionsModel()->index(i, 1);
+			m_vars->modify(idx.sibling(i,0).data().toString(), Analitza::Expression(idx.data().toString()));
+		}
+	} //else TODO: figure out how to control a "rename"
 }
 
 Analitza::Variables* KAlgebraMobile::variables() const { return m_vars; }
-
-QColor randomFunctionColor() { return QColor::fromHsv(qrand()%255, 255, 225); }
-
-QStringList KAlgebraMobile::addFunction(const QString& expression, double up, double down)
-{
-	FunctionsModel* model=functionsModel();
-	
-	Analitza::Variables* vars = variables();
-	Analitza::Expression e(expression, Analitza::Expression::isMathML(expression));
-	QString fname;
-	do {
-		fname = model->freeId();
-	} while(vars->contains(fname));
-	QColor fcolor = randomFunctionColor();
-	
-	QStringList err;
-	Function f(fname, e, vars, fcolor, up,down);
-	
-	if(f.isCorrect()) {
-		bool b = model->addFunction(f);
-		Q_ASSERT(b);
-	} else
-		err = f.errors();
-	
-	return err;
-}
-
